import logging
import time
from threading import *
from AudioSource import *
import baco

class LineInAudioSource(AudioSource):
    def __init__(self, sourceId, name, volume, parameters, audioDev, capDev):
        AudioSource.__init__(self, sourceId, name, volume, parameters, audioDev)

        self.audioDevice = AudioSource.getAudioDev(self)
        self.capDev = capDev
        self.equalizer = None
        self.equalizerConfig = AudioSource.getParameter(self, "equalizer")
        self.trigger = AudioSource.getParameter(self, "trigger")
        self.triggerValue = AudioSource.getParameter(self, "trigger_value")

        self.outputFormat = baco.FrameFormat(baco.ChannelLayout.STEREO, baco.SampleFormat.I16, 44100)

        self.lock = Lock()

        self.logger = logging.getLogger('ipspeaker')
        self.externalCallback = None
        self.alive            = False
        self.startPlayer      = False
        self.stopPlayer       = False
        self.stopMonitor      = False
        self.stopReading      = False
        self.decoderChain     = None

    def runPlayer(self):
        while not self.stopPlayer:
            streamInput = baco.StreamInput()
            res = streamInput.open(self.capDev, "alsa", {"sample_rate": "44100", "channels": "2"})

            self.stopReading = False

            if res == baco.ReturnCode.OK:
                srcInfo = streamInput.getSourceInfo()
                config = baco.Config(srcInfo.getStream(0), self.outputFormat)
                config.inputFifoDuration = 0
                config.outputFifoDuration = 0
                if self.trigger: # On Audio Level
                    levelMeter = baco.LevelMeterConfig
                    levelMeter.enable = True
                    levelMeter.measWindow = 200
                    config.levelMeterConfig = levelMeter
                self.decoderChain = baco.DecoderChain(config)
                self.decoderChain.mute(True)
                self.decoderChain.setGain(1, 0, 0)
                self.decoderChain.setGainMask(1)
                self.setEqualizer()
                alsaSink = baco.AlsaSink(self.audioDevice, self.outputFormat, 0, 0)
                if self.equalizer is not None:
                    self.decoderChain.attachSink(self.equalizer)
                    self.equalizer.setFrameSink(alsaSink)
                    if self.isEqualizerConfEnabled():
                        self.equalizer.enable(True)
                else:
                    self.decoderChain.attachSink(alsaSink)
                self.decoderChain.startOutput()

                if self.trigger:
                    self.stopMonitor = False
                    self.audioLevelThread = Thread(target=self.monitorAudioLevel, daemon=True)
                    self.audioLevelThread.start()

                if not self.trigger and not self.alive: # Always ON
                    self.alive = True
                    if self.externalCallback is not None:
                        self.externalCallback(self, self.alive)

                while not self.stopReading:
                    res = streamInput.readPacket(self.decoderChain)
                    if res != baco.ReturnCode.OK:
                        print("Read Packet Error")
                        self.stopReading = True
                        self.stopMonitor = True

            if self.alive:
                self.alive = False
                if self.externalCallback is not None:
                    self.externalCallback(self, self.alive)

    def monitorAudioLevel(self):
        while not self.stopMonitor:
            if self.decoderChain is not None:
                self.logger.info(f"Audio Levels:{self.decoderChain.getAudioLevels()}")
            time.sleep(1)

    def connect(self):
        with self.lock:
            if not self.startPlayer:
                self.logger.info(f"Connecting source id={self.id}, name={self.name}")
                self.startPlayer = True
                self.playerThread = Thread(target=self.runPlayer, daemon=True)
                self.playerThread.start()

    def disconnect(self):
        self.stopAudio()

    def shutdown(self):
        self.logger.info("Shutdown Line In audio source")
        self.disconnect()
        self.stopReading = True
        self.startPlayer = False
        self.alive = False

    def playAudio(self):
        with self.lock:
            if self.decoderChain is not None:
                self.logger.info(f"Playing source id={self.id}, name={self.name}")
                self.decoderChain.mute(False)

    def stopAudio(self):
        with self.lock:
            self.logger.info(f"Muting source id={self.id}, name={self.name}")
            if self.decoderChain is not None:
                self.decoderChain.mute(True)

    def reportStatus(self):
        sourceDict = {}
        sourceDict["id"] = self.id
        return sourceDict

    def setCallback(self, callback):
        self.externalCallback = callback

    def updateEqualizerConfig(self, name, param, value):
        for filter in self.equalizerConfig['config']:
            if filter['name'] == name:
                filter[param] = str(value)

    def setEqualizer(self):
        try:
            if "config" in self.equalizerConfig and len(self.equalizerConfig['config']) > 0:
                self.logger.info(f"Set Equalizer configuration: {self.equalizerConfig['config']}")
                self.equalizer = baco.Equalizer(self.outputFormat,self.equalizerConfig['config'])
        except Exception as e:
            self.logger.error(e)

    def isEqualizerConfEnabled(self):
        try:
            self.logger.warning(self.equalizerConfig['enable'])
            if self.equalizer is not None and self.equalizerConfig['enable'] == "true":
                self.logger.info("Enable Equalizer")
                return True
            else:
                self.logger.info("Equalizer is disabled (default config)")
                return False
        except Exception as e:
            self.logger.error(e)
            return False

    def setEqualizerParameter(self, config):
        with self.lock:
            try:
                if "config" in self.equalizerConfig and len(self.equalizerConfig['config']) > 0 and any(d["name"] == config["name"] for d in self.equalizerConfig['config']):
                    if "f" in config:
                        if self.equalizer is not None:
                            self.equalizer.setFrequency(config["name"],config["f"])
                        self.updateEqualizerConfig(config['name'],'f',config['f'])
                        self.logger.info(f"Set Filter Central Frequency: {config}")
                    elif "w" in config:
                        if self.equalizer is not None:
                            self.equalizer.setQFactor(config["name"],config["w"])
                        self.updateEqualizerConfig(config['name'],'w',config['w'])
                        self.logger.info(f"Set Filter Q-factor: {config}")
                    elif "g" in config:
                        if self.equalizer is not None:
                            self.equalizer.setGain(config["name"],config["g"])
                        self.updateEqualizerConfig(config['name'],'g',config['g'])
                        self.logger.info(f"Set Filter gain: {config}")
                    else:
                        self.logger.error(f"Set Equalizer parameter:{config}")
                        return False
                    return True

                return False
            except Exception as e:
                self.logger.error(f"Set Equalizer parameter Exception :{e}")
                return False

    def enableEqualizer(self, enable):
        with self.lock:
            try:
                if self.equalizer is not None:
                    self.equalizer.enable(enable)
                self.equalizerConfig['enable'] = enable
                self.logger.info(f"Equalizer enabled {enable}")
            except Exception as e:
                self.logger.error(e)