########################################################################################
#
#   LocalFileAudioSource.py
#
#   Plays a Local Audio File when it receives an UDP command with the Audio File Name
#
#   Config:
#           port - UDP port to receive command
#
#   UDP Command:
#       "FILEPLAY=<filename>\n" , where filename: "Announcement.mp3"
#       "FILESTOP"
#
#   UDP Response:
#       "OK\r\n"
#       "ERROR\r\n"
#       "FILE_NOT_FOUND\r\n"
#       "ALREADY_PLAYING\r\n"
#
########################################################################################
import socket
import os
import logging
import json
import re
from AudioSource import *
from threading import *
import baco

class LocalFileAudioSource(AudioSource):

    def __init__(self, sourceId, name, volume, parameters, audioDev):
        AudioSource.__init__(self, sourceId, name, volume, parameters, audioDev)

        self.audioDevice = audioDev
        self.volume = volume
        self.alive = False
        self.externalCallback = None
        self.running = False
        self.lock = Lock()

        self.command_address = "0.0.0.0"
        self.command_port = str(AudioSource.getParameter(self, "port"))

        self.playbackLoop = False
        if AudioSource.getParameter(self, "loop"):
            self.playbackLoop = True

        self.message_ok = "OK\r\n".encode('utf-8')
        self.message_error = "ERROR\r\n".encode('utf-8')
        self.message_fileNotFound = "FILE_NOT_FOUND\r\n".encode('utf-8')
        self.message_alreadyPlaying = "ALREADY_PLAYING\r\n".encode('utf-8')

        self.player = None
        self.audioFilePlaying = ""
        self.url_audio = None
        # list of internal file names and ids
        self.audioIdList = "/mnt/data/config/audioFiles.json"
        # internal audio folder stores ids of audio files (not filenames)
        self.audio_folder = "/mnt/data/AudioFiles/"
        # pendrive folder stores audio file names
        self.pendrive_folder = "/run/media/sda1/triggered_files/"

        self.logger = logging.getLogger('ipspeaker')

        self.start()

    def start(self):
        self.lock.acquire()
        try:
            if self.running == False:
                # initialize Thread for receiving Commands
                self.udp_thread = Thread(target=self.rcv_command)
                self.udp_thread.daemon = True
                self.stopRcvCommands = False
                self.udp_thread.start()
                self.running = True
        finally:
            self.lock.release()

    def rcv_command(self):
        self.sock_command = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.logger.info("UDP socket : address:{} port:{} ".format(self.command_address, self.command_port))
        self.sock_command.bind((self.command_address, int(self.command_port)))

        while self.stopRcvCommands == False:
            data, addr = self.sock_command.recvfrom(1024)
            self.logger.info("UDP Message: {} ".format(data))
            # check only for \n at the end of string
            match = re.search(r'(\n)$', str(data,'utf-8'))
            if match:
                # analyse command
                match = re.search(r'(\w+)', str(data,'utf-8'))
                if match:
                    if match.group(1) == "FILEPLAY":
                        # try to match command syntax, ex:('FILEPLAY', 'teste', 'mp3', '\n')
                        match = re.match(re.compile(r"^(\w+)=(?P<filename>\w+.*)(\n)"),str(data, 'utf-8'))

                        if match:
                            audiofile = match.group("filename") #+ "." + match.group("extension")
                            self.logger.info("FILEPLAY: Audio File: \"{}\"".format(audiofile))
                            fileID = self.__getFileId(audiofile)
                            if fileID != "":
                                url = str(self.audio_folder) + fileID
                            else:
                                url = str(self.pendrive_folder) + audiofile
                                if not os.path.isfile(url):
                                    url = None

                            self.url_audio = url

                            if self.url_audio is None:
                                self.sock_command.sendto(self.message_fileNotFound, addr)
                                self.logger.error("Audio File not found!")
                            else:
                                if self.alive == False:
                                    self.sock_command.sendto(self.message_ok, addr)
                                    self.logger.info("URL_AUDIO: {}".format(self.url_audio))
                                    self.player = baco.SimplePlayer(self.url_audio, self.audioDevice, 44100)
                                    self.player.setReadyThreshold(500)
                                    if self.playbackLoop:
                                        self.player.setPlaybackLoop(True)
                                    self.player.setCallback(self.checkStateCallback)
                                    self.setEqualizer()
                                    self.audioFilePlaying = audiofile
                                    self.player.connectSource()
                                    self.logger.info(f"Connecting source id={self.id}, name={self.name}")
                                else:
                                    self.sock_command.sendto(self.message_alreadyPlaying, addr)
                                    self.logger.error("Already set a local file for playing")

                        else:
                            self.sock_command.sendto(self.message_error, addr)
                            self.logger.error("Wrong FILEPLAY command syntax!")

                    elif match.group(1) == "FILESTOP":
                        if self.alive:
                            if self.player is not None and self.playbackLoop:
                                self.player.setPlaybackLoop(False)
                            self.alive = False
                            self.restoreVolume()
                            if self.externalCallback is not None:
                                self.externalCallback(self, self.alive)
                            self.logger.info("Stop Playing Local Audio File...")
                        self.sock_command.sendto(self.message_ok, addr)
                    else:
                        self.sock_command.sendto(self.message_error, addr)
                        self.logger.error("Unkown UDP command!")

                else:
                    self.sock_command.sendto(self.message_error, addr)
                    self.logger.error("Wrong UDP command syntax!")

            else:
                self.logger.error("Received Unexpected UDP message!!!")

    def __getFileId(self, filename):
        try:
            res = ""
            with open(self.audioIdList) as f:
                data = json.load(f)
                for index in data:
                    if index["filename"] == filename:
                        return index["file_id"]
            return res
        except Exception as e:
            self.logger.error(f"getFileId Exception {e}")
            return res

    def connect(self):
        pass

    def disconnect(self):
        self.stopAudio()

    def shutdown(self):
        self.logger.info(f"Shutdown source id={self.id}, name={self.name}")
        with self.lock:
            if self.player is not None and self.player.isConnected():
                self.player.disconnect()
            self.stopRcvCommands = True
            self.sock_command.close()
            self.running = False

    def playAudio(self):
        with self.lock:
            self.logger.info(f"Playing source id={self.id}, name={self.name}")
            if self.player is not None:
                self.player.connectAudio()

    def stopAudio(self):
        with self.lock:
            self.logger.info(f"Muting source id={self.id}, name={self.name}")
            if self.player is not None:
                self.player.disconnectAudio()

    def reportStatus(self):
        sourceDict = {}
        sourceDict["id"] = self.id
        sourceDict["playing"] = self.audioFilePlaying
        return sourceDict

    def setCallback(self, callback):
        self.externalCallback = callback

    def restoreVolume(self):
        # when this is triggered by rest API the volume may change
        # it needs to be restored after playing the file
        self.logger.info(f"Restore volume for source id={self.id}, name={self.name}")
        if self.id > 0:  # skip internal local file source
            AudioSource.setVolume(self, self.volume)

    def checkStateCallback(self, alive):
        self.logger.debug(f"Source {self.id}, alive {alive}")
        self.alive = alive
        if not self.alive:
            self.restoreVolume()
        if self.externalCallback is not None:
            self.externalCallback(self, alive)

    def setEqualizer(self):
        with self.lock:
            AudioSource.setEqualizer(self, self.player)

    def setEqualizerParameter(self, config):
        with self.lock:
            return AudioSource.setEqualizerParameter(self, self.player, config)

    def enableEqualizer(self, enable):
        with self.lock:
            AudioSource.enableEqualizer(self, self.player, enable)
