import json
import logging
import os
import socket
import threading

from pathlib import Path

from barix.web.utils import readFromJSONFile
from barix.web.stored_files import getDirectoryContentTree, sortDirectoryTreeList
from barix.web.audio_files import removeAudioFiles, getAudioFileDuration
from barix.web.exceptions import FileIsPlayingError, InvalidAudioFileError

from scheduler import getSchedulerConfig, setSchedulerConfig
from ipformer import getSourcesStatus

log = logging.getLogger('flask-backend')

AUDIO_FILES_JSON_FILE = "/mnt/data/config/audioFiles.json"
USB_FILES_JSON_FILE = "/tmp/usbFiles.json"
PLAYLISTS_FILE_PATH = "/mnt/data/config/playlists.json"
AUDIO_FILES_DIR = "/mnt/data/AudioFiles"
USB_MOUNTED_DIR = "/run/media/sda1"
#AUDIO_FILES_FORMAT = ["mp3", "aac", "flac", "ac3", "dts", "pcm", "vorbis"] # wav -> pcm; ogg -> vorbis; m4a -> aac
AUDIO_FILES_FORMAT = ["mp3","aac","flac","ac3","dts","wav","ogg","m4a","zip"]

def getLocalAudioFiles():
    try:
        localAudioFiles = readFromJSONFile(AUDIO_FILES_JSON_FILE)
    except FileNotFoundError:
        localAudioFiles=[]
    except Exception as e:
        log.error(e)
        raise e
    return localAudioFiles

def getUSBAudioFiles():
    try:
        p = Path(USB_MOUNTED_DIR)
        if p.exists():
            if os.path.isfile(USB_FILES_JSON_FILE):
                usbAudioFiles = readFromJSONFile(USB_FILES_JSON_FILE)
            else:
                usbAudioFiles = getUSBFiles()
        else:
            usbAudioFiles = None
    except Exception as e:
        log.error(e)
        raise e
    else:
        return usbAudioFiles

def getUSBFiles():
    try:
        p = Path(USB_MOUNTED_DIR)
        if p.exists():
            audioFilesAllowedFormats = [x for x in AUDIO_FILES_FORMAT if x!='zip']
            usbAudioFiles = getDirectoryContentTree(USB_MOUNTED_DIR, accepted_file_formats = audioFilesAllowedFormats)
            # remove directories that don't have audio files
            cleanDirectoryTree(usbAudioFiles)
            # sort directory tree
            usbAudioFiles = sortDirectoryTreeList(usbAudioFiles)
            appendParamsToListElements(usbAudioFiles, ['duration'])
        else:
            usbAudioFiles = None
    except Exception as e:
        log.error(e)
        raise e
    else:
        return usbAudioFiles

"""
cleanDirectoryTree removes empty directory entries from the treeList provided
treeList is a list of dictionaries, where each dict represents an entry from the directory
It can be either a file or a directory. 
If it is a directory, it will have a key 'children' which value is a treeList.
"""
def cleanDirectoryTree(treeList):
    try:
        elemsToDelete = []
        for elem in treeList:
            if 'children' in elem.keys():
                if len(elem['children']) == 0:
                    # treeList.remove(elem)
                    elemsToDelete.append(elem)
                else:
                    cleanDirectoryTree(elem['children'])
                    if len(elem['children']) == 0:
                        # treeList.remove(elem)
                        elemsToDelete.append(elem)
        for elem in elemsToDelete:
            treeList.remove(elem)
    except Exception as e:
        log.error(e)
        raise e

def appendParamsToListElements(elemsList, paramsList, treePath=""):
    try:
        for elem in elemsList:
            if "children" in elem.keys():
                currentPath = os.path.join(treePath, elem["name"])
                appendParamsToListElements(elem["children"], paramsList, currentPath)
            else: # file
                for param in paramsList:
                    if param == "duration":
                        try:
                            elem["duration"] = getAudioFileDuration(os.path.join(USB_MOUNTED_DIR, treePath, elem["name"]))
                        except InvalidAudioFileError:
                            log.error('InvalidAudioFileError')
                            #TODO: remove from list
    except Exception as e:
        log.error(e)
        raise e

def getPlaylists():
    playlists = {}
    try:
        if (os.path.exists(PLAYLISTS_FILE_PATH)):
            playlists = readFromJSONFile(PLAYLISTS_FILE_PATH)
    except Exception as e:
        log.error(e)
        raise e
    else:
        return playlists

def deleteAudioFiles(filesIdList):
    try:
        # first, check if file is playing (can't delete a file that is playing)
        sourcesStatus = getSourcesStatus()
        if "status" in sourcesStatus:
            for status in sourcesStatus["status"]:
                if status["priority"] == 0 and status["type"] == "local_file":
                    if status["state"] == "playing":
                        raise FileIsPlayingError
        # then, remove file from playlists and scheduler if it is being used there
        playlistsData = getPlaylists()
        schedulerData = getSchedulerConfig()
        for fileId in filesIdList:
            for playlistId in playlistsData:
                if 'filesList' in playlistsData[playlistId].keys():
                    playlistsData[playlistId]["filesList"] = [x for x in playlistsData[playlistId]["filesList"] if x != fileId]
            if 'fixed_time' in schedulerData.keys():
                schedulerData["fixed_time"] = [x for x in schedulerData["fixed_time"] if x["fileId"] != fileId]
            if 'rotational' in schedulerData.keys() and 'filesList' in schedulerData["rotational"].keys():
                schedulerData["rotational"]["filesList"] = [x for x in schedulerData["rotational"]["filesList"] if x["fileId"] != fileId]
        with open(PLAYLISTS_FILE_PATH, 'w') as outfile:
            json.dump(playlistsData, outfile)
        setSchedulerConfig(schedulerData)
        # finally, remove from the audiofiles json file and from storage
        filesList = [os.path.join(AUDIO_FILES_DIR, x) for x in filesIdList]
        res = removeAudioFiles(AUDIO_FILES_JSON_FILE, filesList)
        return res
    except FileIsPlayingError:
        raise FileIsPlayingError
    except Exception as e:
        log.error(e)
        raise e

socketResponse = None
cv = threading.Condition() #conditionVariable

def playAudioFile(audioFileName, ip_addr='127.0.0.1', port=60000):
    global socketResponse, cv
    log.info("Requesting to play \"{}\"".format(audioFileName))
    socketResponse = None
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  # UDP
        t = threading.Thread(target=waitForResult,
                             args=(sock,))
        t.start()
        message = "FILEPLAY={}\n".format(audioFileName)
        sock.sendto(bytes(message, "utf-8"), (ip_addr, port))
        with cv:
            while socketResponse is None:
                cv.wait()
        return socketResponse
    except Exception as e:
        log.error(e)
        raise e

def waitForResult(sock):
    global socketResponse
    response = sock.recvfrom(1024)
    socketResponse = response[0].decode("utf-8").strip('\r\n')
    with cv:
        cv.notify_all()

def stopPlayingAudioFile(ip_addr='127.0.0.1', port=60000):
    log.info("Stop playing audio file")
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  # UDP
        message = "FILESTOP\n"
        sock.sendto(bytes(message, "utf-8"), (ip_addr, port))
    except Exception as e:
        log.error(e)
        raise e