import json
import logging
import sys
from flask import Blueprint, Response, request

from uci import UciException
from barix import HTTPRequestsAPI

from barix.web.uci import getValueOfUci, setUciConfigs
from barix.web.auth import login_required
from barix.web.system_funcs import setVolume, setMixerVolume

from ipformer import MAX_N_SOURCES, getIPFormerSources, setIpFormerSources, getSourcesStatus, setEqualizerConfigInUci, InvalidSourceConfigurationError

sys.path.insert(0, '/barix/apps/ipac/api/')
from restAPI import validateSourcesConfiguration

log = logging.getLogger('flask-backend')

bp = Blueprint('ipformer_API', __name__)

@bp.route('/api/rest/v1/ipformer/sources', methods=['GET'])
@login_required
def handleGetIPFormerSources():
    try:
        sourcesList = getIPFormerSources()
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        #log.debug("sources: {}".format(json.dumps(sourcesList)))
        return Response(json.dumps(sourcesList), status=200)


@bp.route('/api/rest/v1/ipformer/sources', methods=['PUT'])
@login_required
def handleSetIPSpeakerSources():
    try:
        data = json.loads(request.data)
        log.debug("Source configurations to set: {}".format(data))
    except Exception as e:
        log.error(e)
        return Response("Invalid request body", status=400)
    try:
        # validate sources configuration
        valid, description = validateSourcesConfiguration(data)
        if not valid:
            log.error("Invalid configuration: {}".format(description))
            raise InvalidSourceConfigurationError
        setIpFormerSources(data["sources"])
    except InvalidSourceConfigurationError:
        return Response("Invalid configuration", status=400)
    except UciException as e:
        return Response("Invalid " + e, status=400)
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        return Response(status=200)


@bp.route('/api/rest/v1/ipformer/speaker-status', methods=['GET'])
@login_required
def handleGetSpeakerStatus():
    try:
        sourcesStatus = getSourcesStatus()
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        return Response(json.dumps(sourcesStatus), status=200)


@bp.route('/api/rest/v1/ipformer/dante-status', methods=['GET'])
@login_required
def handleGetDanteDeviceStatus():
    try:
        responseDict = { "device" : "", "clocking": {}}
        host_addr = "127.0.0.1:8088"

        url = "http://" + host_addr + "/api/v1/dante/device"
        try:
            r = HTTPRequestsAPI.get(url)
            if r.status == 200:
                info = r.read()
                enc = r.info().get_content_charset('utf-8')
                responseDict = json.loads(info.decode(enc))
            else:
                log.error("Error {}".format(r))
        except Exception as e:
            log.error(e)
            raise e

        url = "http://" + host_addr + "/api/v1/dante/clocking"
        try:
            r = HTTPRequestsAPI.get(url)
            if r.status == 200:
                info = r.read()
                enc = r.info().get_content_charset('utf-8')
                responseDict["clocking"] = json.loads(info.decode(enc))
            else:
                log.error("Error {}".format(r))
        except Exception as e:
           log.error(e)
           raise e

        return responseDict

    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        return Response(json.dumps(sourcesStatus), status=200)


@bp.route('/api/rest/v1/ipformer/source/<source_id>/equalizer', methods=['PUT'])
@login_required
def handleSetSourceEqualizerRequest(source_id):
    try:
        try:
            data = json.loads(request.data)
            log.debug("Source {} configurations to set: {}".format(source_id, data))
        except Exception as e:
            log.error(e)
            return Response("Invalid request body", status=400)

        if not(int(source_id) >=1 and int(source_id) <= MAX_N_SOURCES):
            log.error("Invalid source id {}".format(source_id))
            return Response("Invalid request body", status=400)

        if "enable" not in data.keys() and "config" not in data.keys():
            log.error("Invalid keys {}".format(data.keys()))
            return Response("Invalid request body", status=400)

        if "enable" in data.keys():
            url = "http://127.0.0.1:50500/api/equalizer/enable"
            requestBody = {"sourceId": int(source_id), "enable": data["enable"]}
            try:
                log.info("Requesting {} to {}".format(requestBody, url))
                r = HTTPRequestsAPI.put(url, requestBody)
                if r.status != 200:
                    log.error("Error requesting to {}: {} - {}".format(url, r.status, r.reason))
                    return Response(status=500)
            except Exception as e:
                log.error(e)
                return Response(status=500)
        if "config" in data.keys():
            url = "http://127.0.0.1:50500/api/equalizer/config"
            for filter in data["config"]:
                for param in data["config"][filter]:
                    filterDict = {"name": filter.upper()}
                    filterDict[param] = data["config"][filter][param]
                    requestBody = {"sourceId": int(source_id), "config": filterDict}
                    try:
                        log.info("Requesting {} to {}".format(requestBody, url))
                        r = HTTPRequestsAPI.put(url, requestBody)
                        if r.status != 200:
                            log.error("Error requesting to {}: {} - {}".format(url, r.status, r.reason))
                            return Response(status=500)
                    except Exception as e:
                        log.error(e)
                        return Response(status=500)
    #store in ucis
    except Exception as ex:
        log.error(ex)
        return Response("Ups", status=418)

    try:
        setEqualizerConfigInUci(source_id, data)
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        return Response(status=200)


@bp.route('/api/rest/v1/ipformer/source/<source_id>/volume', methods=['PUT'])
@login_required
def handleSetSourceVolumeRequest(source_id):
    try:
        volumeData = json.loads(request.data)
        for volumeKey in volumeData:
            if 0 > int(volumeData[volumeKey]) > 100:
                return Response("Volume invalid", status=400)
    except Exception as e:
        log.error(e)
        return Response("Bad formatted request body", status=400)
    try:
        for volumeKey in volumeData:
            if volumeKey == 'volume':
                #check if source is configured
                sourceData = getValueOfUci("flexa_ip_former", "source" + source_id, "volume")
                if sourceData != '':
                    uci = "flexa_ip_former.source"+source_id+".volume"
                    setVolume(uci, int(volumeData[volumeKey]))
                # else ignore command since source is not yet configured
            elif volumeKey == 'vol_announcement':
                mixer = 'chan_11'
                setMixerVolume(mixer, int(volumeData[volumeKey]))
                # store value in uci
                sourceData = getValueOfUci("flexa_ip_former", "source" + source_id, "config")
                if sourceData != '':
                    sourceConfig = json.loads(sourceData)
                    sourceConfig[volumeKey] = volumeData[volumeKey]
                    setUciConfigs({"flexa_ip_former.source" + source_id +".config": sourceConfig}, commit=True, restartServices=False,
                                  selectServicesToRestartCallback=None)
            elif volumeKey == 'vol_playlist':
                mixer = 'chan_12'
                setMixerVolume(mixer, int(volumeData[volumeKey]))
                # store value in uci
                sourceData = getValueOfUci("flexa_ip_former", "source" + source_id, "config")
                if sourceData != '':
                    sourceConfig = json.loads(sourceData)
                    sourceConfig[volumeKey] = volumeData[volumeKey]
                    setUciConfigs({"flexa_ip_former.source" + source_id + ".config": sourceConfig}, commit=True,
                                  restartServices=False,
                                  selectServicesToRestartCallback=None)
            else:
                log.error("Invalid key {}".format(volumeKey))
                return Response("Bad formatted request body", status=400)
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        return Response(status=200)
