#!/usr/bin/python3
import logging
from threading import Thread
import time
import baco
from GenericRestAPI import *
from enum import Enum

logger = logging.getLogger('ipspeaker')

"""
Streaming using DEP (Dante Embedded Platform)
Audio flow: loopback2_in (hw:0,1,4) -> dsoundcard alsa Dante plugin -> DEP -> Dante RTP stream 
"""

class DanteAppState(Enum):
    INITIAL = 0
    DANTE_READY = 1

class DanteStreamer:
    def __init__(self):
        self.stopReading    = False
        self.stopWorking    = False
        self.stopPlaying    = False
        self.state          = DanteAppState.INITIAL
        self.bacoDanteApi   = GenericRestAPI(8088)
        self.outputFormat   = None
        self.danteThread    = None
        self.playerThread   = None
        self.currSampleRate = None

    def start(self):
        if self.danteThread is None:
            self.danteThread = Thread(target=self._checkDante, daemon=True)
            self.danteThread.start()

    def _checkDante(self):
        while not self.stopWorking:
            while self.state == DanteAppState.INITIAL and not self.stopWorking:
                danteReady = self.bacoDanteApi.get('/api/v1/dante/ready')
                if "ready" in danteReady and danteReady["ready"]:
                    logger.info("Dante Streamer Initial State - DEP is READY")
                    self.state = DanteAppState.DANTE_READY
                else:
                    logger.warning("Dante Streamer Initial State - DEP is NOT READY")
                    time.sleep(1)

            while self.state == DanteAppState.DANTE_READY and not self.stopWorking:
                danteAudioFormat = self.bacoDanteApi.get('/api/v1/dante/audio_format')
                if "sample_rate" in danteAudioFormat and danteAudioFormat["sample_rate"] in [44100, 48000]:
                    if self.currSampleRate != danteAudioFormat["sample_rate"]:
                        if self.currSampleRate is None:
                            logger.info(f"Sample Rate->{danteAudioFormat['sample_rate']}")
                        else:
                            logger.info(f"Sample Rate changed to {danteAudioFormat['sample_rate']}")
                        self.currSampleRate = danteAudioFormat["sample_rate"]
                        self.outputFormat = baco.FrameFormat(baco.ChannelLayout.STEREO, baco.SampleFormat.I32, self.currSampleRate)
                        self._stopPlayer()
                        self._startPlayer()
                    else:
                        time.sleep(1)
                else:
                    logger.warning("Error reading Dante Sample Rate => Dante State Initial")
                    self.state = DanteAppState.INITIAL

    def _startPlayer(self):
        if self.playerThread is None:
            logger.info("Start Player")
            self.stopReading = False
            self.stopPlaying = False
            self.playerThread = Thread(target=self._runPlayer, daemon=True)
            self.playerThread.start()

    def _stopPlayer(self):
        if self.playerThread is not None:
            logger.info("Stop Player")
            self.stopReading = True
            self.stopPlaying = True
            self.playerThread.join()
            self.playerThread = None

    def _runPlayer(self):
        while not self.stopPlaying:
            streamInput = baco.StreamInput()
            res = streamInput.open("plug:loopback_in","alsa",{"sample_rate": "44100","channels": "2"})

            if res == baco.ReturnCode.OK:
                srcInfo = streamInput.getSourceInfo()
                config = baco.Config(srcInfo.getStream(0), self.outputFormat)
                config.inputFifoDuration = 0
                config.outputFifoDuration = 50
                decoderChain = baco.DecoderChain(config)
                decoderChain.mute(False)
                decoderChain.setGain(1,0,0)
                decoderChain.setGainMask(1)
                alsaSink = baco.AlsaSink("dsoundcard", self.outputFormat, 0, 0)
                decoderChain.attachSink(alsaSink)
                decoderChain.startOutput()
                self.stopReading = False
                while not self.stopReading:
                    res = streamInput.readPacket(decoderChain)
                    if res != baco.ReturnCode.OK:
                        print("Read Packet Error")
                        self.stopReading = True

    def stop(self):
        self._stopPlayer()
        self.stopWorking = True
        self.danteThread.join()
        self.danteThread = None
        self.state = DanteAppState.INITIAL
